# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['parserutils', 'parserutils.tests']

package_data = \
{'': ['*'], 'parserutils.tests': ['data/*']}

install_requires = \
['defusedxml>=0.7.1,<0.8.0', 'python-dateutil>=2.8.2,<3.0.0']

setup_kwargs = {
    'name': 'parserutils',
    'version': '2.0',
    'description': 'A collection of performant parsing utilities',
    'long_description': "# parserutils\n\n[![Build Status](https://api.travis-ci.com/consbio/parserutils.png?branch=main)](https://app.travis-ci.com/github/consbio/parserutils)\n[![Coverage Status](https://coveralls.io/repos/github/consbio/parserutils/badge.svg?branch=main)](https://coveralls.io/github/consbio/parserutils?branch=main)\n\nThis is a library of utility functions designed to make a developer's life easier.\n\nThe functions in this library are written to be both performant and Pythonic, as well as compatible with Python 3.6 through 3.9.\nThey are both documented and covered thoroughly by unit tests that fully describe and prove their behavior.\n\nIn general, my philosophy is that utility functions should be fast and handle edge cases so the caller doesn't have to take all kinds of precautions or do type checking on results.\nThus, in this library, if None will break a function it is simply returned as is; if there's nothing to do for a value, the result is returned without processing; otherwise, values are either processed successfully or a standard exception is returned.\n\nBut this is just a starting point. I welcome feedback and requests for additional functionality.\n\n\n## Installation\nInstall with `pip install parserutils`.\n\n## Usage\n\nHere's what you can do with `dict` objects and other collections.\n```python\nfrom parserutils import collections\n\ncollections.accumulate_items([('key', 'val1'), ('key', 'val2'), ('key', 'val3')])   # {'key': ['val1', 'val2', 'val3']}\ncollections.accumulate_items(\n    [('key1', 'val1'), ('key2', 'val2'), ('key3', 'val3')], reduce_each=True  # {'key1': 'val1', 'key2': 'val2', 'key3': 'val3'}\n)\n\ncollections.setdefaults({}, 'a.b')                         # {'a': {'b': None}}\ncollections.setdefaults({}, ['a.b', 'a.c'])                # {'a': {'b': None, 'c': None}}\ncollections.setdefaults({}, {'a.b': 'bbb', 'a.c': 'ccc'})  # {'a': {'b': 'bbb', 'c': 'ccc'}}\n\ncollections.filter_empty(x for x in (None, [], ['a'], '', {'b'}, 'c'))      # [['a'], {'b'}, 'c']\ncollections.flatten_items(x for x in ('abc', ['a', 'b', 'c'], ('d', 'e')))  # ['abc', 'a', 'b', 'c', 'd', 'e']\n\ncollections.remove_duplicates('abcdefabc')                                   # 'abcdef'\ncollections.remove_duplicates('abcdefabc', in_reverse=True)                  # 'defabc'\ncollections.remove_duplicates(['a', 'b', 'c', 'a'])                          # ['a', 'b', 'c']\ncollections.remove_duplicates(('a', 'b', 'c', 'a'), in_reverse=True)         # ('b', 'c', 'a')\ncollections.remove_duplicates(x for x in 'abca')                             # ['a', 'b', 'c']\ncollections.remove_duplicates((x for x in 'abca'), in_reverse=True)          # ['b', 'c', 'a']\ncollections.remove_duplicates((set(x) for x in 'abca'), is_unhashable=True)  # [{'a'}, {'b'}, {'c'}]\n\ncollections.rindex('aba', 'a')               # 2\ncollections.rindex(['a', 'b', 'a'], 'a')     # 2\ncollections.rindex(('a', 'b', 'a'), 'a')     # 2\ncollections.rindex('xyz', 'a')               # ValueError\ncollections.rindex([x for x in 'xyz'], 'a')  # ValueError\n\ncollections.rfind('aba', 'a')                # 2\ncollections.rfind(['a', 'b', 'a'], 'a')      # 2\ncollections.rfind(('a', 'b', 'a'), 'a')      # 2\ncollections.rindex('xyz', 'a')               # -1\ncollections.rfind([x for x in 'xyz'], 'a')   # -1\n\ncollections.reduce_value(['abc'])          # 'abc'\ncollections.reduce_value(('abc',))         # 'abc'\ncollections.reduce_value({'abc'})          # 'abc'\ncollections.reduce_value('abc')            # 'abc'\ncollections.reduce_value({'a': 'aaa'})     # {'a': 'aaa'}\ncollections.reduce_value([{'a': 'aaa'}])   # {'a': 'aaa'}\ncollections.reduce_value(['a', 'b', 'c'])  # ['a', 'b', 'c']\n\ncollections.wrap_value(['abc'])           # ['abc']\ncollections.wrap_value(('abc',))          # ('abc',)\ncollections.wrap_value('abc')             # ['abc']\ncollections.wrap_value(x for x in 'abc')  # ['a', 'b', 'c']\ncollections.wrap_value({'a': 'aaa'})      # [{'a': 'aaa'}]\ncollections.wrap_value(['a', 'b', 'c'])   # ['a', 'b', 'c']\n```\n\nHere's a little bit about dates and numbers.\n```python\nfrom parserutils import dates\nfrom parserutils import numbers\n\n# Leverages dateutil in general, but also handles milliseconds and provides defaults\n\ndates.parse_dates(None, default='today')  # Today (default behavior)\ndates.parse_dates(None, default=None)     # Returns None\ndates.parse_dates('nope', default=None)   # Returns None\ndates.parse_dates(0)                      # 1970\ndates.parse_dates('<date_format>')        # Behaves as described in dateutil library\n\n# Reliably handles all the usual cases\n\nnumbers.is_number(0)                    # Integer: True\nnumbers.is_number(1.1)                  # Float: True\nnumbers.is_number('2.2')                # String: True\nnumbers.is_number(False)                # Boolean: False by default\nnumbers.is_number(False, if_bool=True)  # Boolean: True if you need it to\nnumbers.is_number(float('inf'))         # Infinite: False\nnumbers.is_number(float('nan'))         # NaN: False\n```\n\nHere's something about string and URL parsing helpers.\n```python\nfrom parserutils import strings\nfrom parserutils import urls\n\n# These string conversions are written to be fast and reliable\n\nstrings.camel_to_constant('toConstant')        # TO_CONSTANT\nstrings.camel_to_constant('XMLConstant')       # XML_CONSTANT\nstrings.camel_to_constant('withNumbers1And2')  # WITH_NUMBERS1_AND2\n\nstrings.camel_to_snake('toSnake')              # to_snake\nstrings.camel_to_snake('withXMLAbbreviation')  # with_xml_abbreviation\nstrings.camel_to_snake('withNumbers3And4')     # with_numbers3_and4\n\nstrings.snake_to_camel('from_snake')              # fromSnake\nstrings.snake_to_camel('_leading_and_trailing_')  # leadingAndTrailing\nstrings.snake_to_camel('extra___underscores')     # extraUnderscores\n\nstrings.find_all('ab??ca??bc??', '??')                         # [2, 6, 10]\nstrings.find_all('ab??ca??bc??', '??', reverse=True)           # [10, 6, 2]\nstrings.find_all('ab??ca??bc??', '??', limit=2, reverse=True)  # [10, 6]\nstrings.find_all('ab??ca??bc??', '??', start=4)                # [6, 10]\nstrings.find_all('ab??ca??bc??', '??', end=8)                  # [2, 6]\nstrings.find_all('ab??ca??bc??', '??', start=4, end=8)         # [6]\n\nstrings.splitany('ab:ca:bc', ',')           # Same as 'ab:ca:bc'.split(':')\nstrings.splitany('ab:ca:bc', ',', 1)        # Same as 'ab:ca:bc'.split(':', 1)\nstrings.splitany('ab|ca:bc', '|:')          # ['ab', 'ca', 'bc']\nstrings.splitany('ab|ca:bc', ':|', 1)       # ['ab', 'ca:bc']\nstrings.splitany('0<=3<5', ['<', '<='])     # ['0', '3', '5']\nstrings.splitany('0<=3<5', ['<', '<='], 1)  # ['0', '3<5']\n\nstrings.to_ascii_equivalent('smart quotes, etc.')  # Replaces with ascii quotes, etc.\n\n# URL manipulation leverages urllib, but spares you the extra code\n\nurls.get_base_url('http://www.params.com?a=aaa')                  # 'http://www.params.com'\nurls.get_base_url('http://www.path.com/test')                     # 'http://www.path.com'\nurls.get_base_url('http://www.path.com/test', include_path=True)  # 'http://www.path.com/test'\nurls.get_base_url('http://www.params.com/test?a=aaa', True)       # 'http://www.params.com/test'\n\nurls.update_url_params('http://www.params.com?a=aaa', a='aaa')  # 'http://www.params.com?a=aaa'\nurls.update_url_params('http://www.params.com?a=aaa', a='xxx')  # 'http://www.params.com?a=xxx'\nurls.update_url_params('http://www.params.com', b='bbb')        # 'http://www.params.com?b=bbb'\nurls.update_url_params('http://www.params.com', c=['c', 'cc'])  # 'http://www.params.com?c=c&c=cc'\n\n# Helpers to parse urls to and from parts: parses path as list and params as dict\nurls.url_to_parts('http://www.params.com/test/path?a=aaa')      # SplitResult(..., path=['test', 'path'], query={'a': 'aaa'})\nurls.parts_to_url(\n    {'netloc': 'www.params.com', 'query': {'a': 'aaa'}          # 'http://www.params.com?a=aaa'\n)\nurls.parts_to_url(\n    urls.url_to_parts('http://www.params.com/test/path?a=aaa')  # 'http://www.params.com/test/path?a=aaa'\n)\n```\n\nFinally, XML parsing is also supported, using the cElementTree and defusedxml libraries for performance and security\n```python\nfrom parserutils import elements\n\n# First convert an XML string to an Element object\nxml_string = '<root><parent><child>one</child><child>two</child><uglyChild>yuck</uglyChild></parent></root>'\nxml_element = elements.get_element(xml_string)\n\n\n# Update the XML string and print it back out\nelements.set_element_text(xml_element, 'parent/child', 'child text')\nelements.set_element_attributes(xml_element, childHas='child attribute')\nelements.remove_element(xml_element, 'parent/uglyChild')\nelements.element_to_string(xml_element)\n\n\n# Conversion from string to Element, to dict, and then back to string\nconverted = elements.element_to_dict(xml_string, recurse=True)\nreverted = elements.dict_to_element(converted)\nreverted = elements.get_element(converted)\nxml_string == elements.element_to_string(converted)\n\n\n# Conversion to flattened dict object\nroot, obj = elements.element_to_object(converted)\nobj == {'root': {'parent': {'child': ['one', 'two'], 'uglyChild': 'yuck'}}}\n\n\n# Read in an XML file and write it elsewhere\nwith open('/path/to/file.xml', 'wb') as xml:\n    xml_from_file = elements.get_element(xml)\n    elements.write_element(xml_from_file, '/path/to/updated/file.xml')\n\n\n# Write a local file from a remote location (via URL)\nxml_from_web = elements.get_remote_element('http://en.wikipedia.org/wiki/XML')\nelements.write_element(xml_from_web, '/path/to/new/file.xml')\n\n\n# Read content at a local file path to a string\nxml_from_path = elements.get_remote_element('/path/to/file.xml')\nelements.element_to_string(xml_from_path)\n```\n",
    'author': 'dharvey-consbio',
    'author_email': 'dani.harvey@consbio.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/consbio/parserutils/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
